@@ -53,6 +53,10 @@ def smooth_step(x):
         return(1.0)
     else:
         return(-2.0*x*x*x+3.0*x*x)
+        
+# Conversion of erf function into CDF format
+def cdf(x):
+    return((1.0+math.erf(x/math.sqrt(2)))/2)
 
 # -------- new step function 2018-09-16 ------
 def stepsWithPeriods(N, period, width, iCurr):
@@ -253,7 +257,7 @@ class Expression:
                 self.l1elem = self.GetNodeOrElement(circuit, ast_exp.fargs[0])
                 self.l2elem = self.GetNodeOrElement(circuit, ast_exp.fargs[1])
                 #self.fargs = [Expression(ast_exp.fargs[2], circuit, phase_dependence)]
-            elif self.fname in ['print','min','max','round']:
+            elif self.fname in ['abs','print','min','max','round']:
                 self.fargs = []
                 for arg in ast_exp.fargs:
                     self.fargs.append(Expression(arg, circuit, phase_dependence))
@@ -294,6 +298,40 @@ class Expression:
                     self.etype = ExpressionAST.EXP_TYPE_INVALID
                     raise Exception("Invalid parameters for " + self.fname + " in " + str(circuit))
                 self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence)]
+            elif self.fname == 'random':
+                self.fargs = []
+            elif self.fname == 'cdf':
+                if len(ast_exp.fargs) == 1:
+                    self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence)]
+                else:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid number of parameters for function cdf() in" + str(circuit))
+                if self.fargs[0].etype == ExpressionAST.EXP_TYPE_INVALID:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid parameter for function cdf() in " + str(circuit))
+            elif self.fname == 'setsmooth':
+                if len(ast_exp.fargs) == 3 and ast_exp.fargs[0].etype == ExpressionAST.EXP_TYPE_IDENT:
+                    ident = ast_exp.fargs[0].ident
+                    if ident in local_externals:
+                        self.set_param = local_externals[ident]
+                    elif ident in local_params:
+                        self.set_param = local_params[ident]
+                    elif ident in psglobals.Parameters:
+                        self.set_param = psglobals.Parameters[ident]
+                    else:
+                        self.etype = ExpressionAST.EXP_TYPE_INVALID
+                        raise Exception("Invalid first parameter for function setsmooth() in" + str(circuit))
+                       
+                    self.fargs = [Expression(ast_exp.fargs[1], circuit, phase_dependence),
+                                  Expression(ast_exp.fargs[2], circuit, phase_dependence)]
+                else:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid number of parameters for function setsmooth() in" + str(circuit))
+                    
+                if self.fargs[0].etype == ExpressionAST.EXP_TYPE_INVALID or \
+                    self.fargs[1].etype == ExpressionAST.EXP_TYPE_INVALID:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid parameters for function setsmooth() in " + str(circuit))
             elif self.fname in dir(math):
                 # One of the math module functions
                 self.func = getattr(math, self.fname)
@@ -604,6 +642,8 @@ class Expression:
                 if self.circuit is not None:
                     self.circuit.UnFreezeJunctions(self.fargs)
                 return(1.0)
+            elif self.fname == 'abs':
+                return(abs(self.fargs[0].Value()))
             elif self.fname == 'min':
                 return(min(q.Value() for q in self.fargs))
             elif self.fname == 'max':
@@ -625,6 +665,13 @@ class Expression:
                     return self.false_exp.Value()
             elif self.fname == 'avert':
                 return self.avert.step(psglobals.CurrentTime, self.fargs[0].Value())
+            elif self.fname == 'random':
+                return random.random()
+            elif self.fname == 'cfd':
+                return cdf(self.fargs[0].Value())
+            elif self.fname == 'setsmooth':
+                self.set_param.SetValueSmooth(self.fargs[0].Value(), self.fargs[1].Value())
+                return(1.0)
             elif self.fname == 'pyeval':
                 return eval(self.fargs[0].sconst_val)
             elif self.func is not None:
